@@ -252,7 +252,7 @@ class Ropper(object):
         return toReturn
 
     def _searchPopPopRet(self, section, binary):
-        if binary.arch != x86:
+        if not isinstance(binary.arch, x86.__class__ ):
             raise NotSupportedError(
                 'Wrong architecture, \'pop pop ret\' is only supported on x86')
 
@@ -271,7 +271,7 @@ class Ropper(object):
                     toReturn.append(pprg)
         return toReturn
 
-    def searchGadgets(self, binary, instructionCount=5, gtype=GadgetType.ALL):
+    def searchGadgets(self, binary, instructionCount=5, gtype=GadgetType.ALL, multiprocessing=False):
         Gadget.IMAGE_BASES[binary.checksum] = binary.imageBase
         gadgets = []
         for section in binary.executableSections:
@@ -280,7 +280,7 @@ class Ropper(object):
             if self.__callback:
                 self.__callback(section, None, 0)
 
-            if sys.platform.startswith('win'):
+            if not multiprocessing:
                 newGadgets = self._searchGadgetsSingle(section=section, binary=binary, instruction_count=instructionCount, gtype=gtype)
             else:
                 newGadgets = self._searchGadgetsForked(section=section, binary=binary, instruction_count=instructionCount, gtype=gtype)
@@ -293,7 +293,9 @@ class Ropper(object):
 
         toReturn = []
         code = bytes(bytearray(section.bytes))
-        offset = section.offset
+        # TODO: Another solution should be used here. This is a hack for compatibility reasons. to resolve the gadget address calculation of segments of elf files have a different base address if calculated segment.virtualAddress - segment.offset 
+        offset = section.offset - (binary.imageBase - (section.virtualAddress - section.offset))
+        #offset = section.offset
 
         arch = binary.arch
 
@@ -364,8 +366,11 @@ class Ropper(object):
         for cpu in range(process_count):
             ending_queue.put(None)
 
+        # TODO: Another solution should be used here. This is a hack for compatibility reasons. to resolve the gadget address calculation of segments of elf files have a different base address if calculated segment.virtualAddress - segment.offset 
+        offset = section.offset - (binary.imageBase - (section.virtualAddress - section.offset))
+
         for cpu in range(process_count):
-            processes.append(Process(target=self.__gatherGadgetsByEndings, args=(tmp_code, arch, binary.checksum, section.name, section.offset, ending_queue, gadget_queue, instruction_count), name="GadgetSearch%d"%cpu))
+            processes.append(Process(target=self.__gatherGadgetsByEndings, args=(tmp_code, arch, binary.checksum, section.name, offset, ending_queue, gadget_queue, instruction_count), name="GadgetSearch%d"%cpu))
             processes[cpu].daemon=True
             processes[cpu].start()
 
@@ -422,7 +427,9 @@ class Ropper(object):
                 none_count = 0
 
                 for x in range(0, index+1, arch.align):
-                    code_part = tmp_code[index - x:index + ending[1]]
+                    start = index - x
+                    end = index + ending[1] + arch.align if (arch.hasBranchDelaySlot and index + ending[1] + arch.align < len(tmp_code)) else index + ending[1]
+                    code_part = tmp_code[start:end]
                     gadget, leng = self.__createGadget(arch, code_part, offset + offset_tmp - x , ending, fileName, sectionName)
                     if gadget:
                         if leng > instruction_count:
@@ -445,7 +452,8 @@ class Ropper(object):
     def __createGadget(self, arch, code_str, codeStartAddress, ending, binary=None, section=None):
         gadget = Gadget(binary, section, arch)
         hasret = False
-
+        if codeStartAddress == 0x0000000001b34d74:
+            print("found")
         disassembler = self.__getCs(arch)
 
         for i in disassembler.disasm(code_str, codeStartAddress):
@@ -456,7 +464,7 @@ class Ropper(object):
                 gadget.append(
                     i.address, i.mnemonic,i.op_str, bytes=i.bytes)
 
-            if hasret or i.mnemonic in arch.badInstructions:
+            if (hasret and not arch.hasBranchDelaySlot) or i.mnemonic in arch.badInstructions:
                 break
 
 
